==> omu-0.2.5.tar.gz: omu-0.2.5/src/omu/event_emitter.py <==
from __future__ import annotations

import asyncio
from typing import Callable, List, Self

from omu.helper import Coro


class EventEmitter[**P]:
    def __init__(
        self,
        on_subscribe: Callable[[], None] | Coro[[], None] | None = None,
        on_empty: Callable[[], None] | Coro[[], None] | None = None,
    ) -> None:
        self.on_subscribe = on_subscribe
        self.on_empty = on_empty
        self._listeners: List[Callable[P, None] | Coro[P, None]] = []

    @property
    def empty(self) -> bool:
        return len(self._listeners) == 0

    def subscribe(self, listener: Callable[P, None] | Coro[P, None]) -> None:
        if listener in self._listeners:
            raise ValueError("Listener already subscribed")
        if self.on_subscribe and len(self._listeners) == 0:
            coroutine = self.on_subscribe()
            if asyncio.iscoroutine(coroutine):
                asyncio.create_task(coroutine)
        self._listeners.append(listener)

    def unsubscribe(self, listener: Callable[P, None] | Coro[P, None]) -> None:
        if listener not in self._listeners:
            return
        self._listeners.remove(listener)
        if self.on_empty and len(self._listeners) == 0:
            coroutine = self.on_empty()
            if asyncio.iscoroutine(coroutine):
                asyncio.create_task(coroutine)

    def __iadd__(self, listener: Callable[P, None] | Coro[P, None]) -> Self:
        self.subscribe(listener)
        return self

    def __isub__(self, listener: Callable[P, None] | Coro[P, None]) -> Self:
        self.unsubscribe(listener)
        return self

    async def emit(self, *args: P.args, **kwargs: P.kwargs) -> None:
        for listener in tuple(self._listeners):
            if asyncio.iscoroutinefunction(listener):
                await listener(*args, **kwargs)
            else:
                listener(*args, **kwargs)

    __call__ = emit

==> omu-0.3.2.tar.gz: omu-0.3.2/src/omu/event_emitter.py <==
from __future__ import annotations

import asyncio
from collections.abc import Callable
from typing import Self

from omu.helper import Coro

type Unlisten = Callable[[], None]


class EventEmitter[**P]:
    def __init__(
        self,
        on_subscribe: Callable[[], None] | Coro[[], None] | None = None,
        on_empty: Callable[[], None] | Coro[[], None] | None = None,
    ) -> None:
        self.on_subscribe = on_subscribe
        self.on_empty = on_empty
        self._listeners: list[Callable[P, None] | Coro[P, None]] = []
        self.closed = False

    @property
    def empty(self) -> bool:
        return len(self._listeners) == 0

    def close(self) -> None:
        self.closed = True
        self._listeners.clear()

    def listen(self, listener: Callable[P, None] | Coro[P, None]) -> Unlisten:
        if self.closed:
            raise ValueError("EventEmitter is closed")
        if listener in self._listeners:
            raise ValueError("Listener already subscribed")
        if self.on_subscribe and len(self._listeners) == 0:
            coroutine = self.on_subscribe()
            if asyncio.iscoroutine(coroutine):
                asyncio.create_task(coroutine)
        self._listeners.append(listener)
        return lambda: self.unlisten(listener)

    def unlisten(self, listener: Callable[P, None] | Coro[P, None]) -> None:
        if listener not in self._listeners:
            return
        self._listeners.remove(listener)
        if self.on_empty and len(self._listeners) == 0:
            coroutine = self.on_empty()
            if asyncio.iscoroutine(coroutine):
                asyncio.create_task(coroutine)

    async def emit(self, *args: P.args, **kwargs: P.kwargs) -> None:
        if self.closed:
            raise ValueError("EventEmitter is closed")
        for listener in tuple(self._listeners):
            if asyncio.iscoroutinefunction(listener):
                await listener(*args, **kwargs)
            else:
                listener(*args, **kwargs)

    def __iadd__(self, listener: Callable[P, None] | Coro[P, None]) -> Self:
        self.listen(listener)
        return self

    __call__ = emit
